from __future__ import unicode_literals

from allauth.account.adapter import DefaultAccountAdapter
from allauth.utils import build_absolute_uri
from django.conf import settings
from django.contrib.sites.shortcuts import get_current_site
from django.core.mail import EmailMessage, EmailMultiAlternatives
from django.http import HttpResponseRedirect
from django.shortcuts import resolve_url
from django.template import TemplateDoesNotExist
from django.template.loader import render_to_string
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _


class MyAccountAdapter(DefaultAccountAdapter):
    def get_signup_redirect_url(self, request):
        return resolve_url(settings.SIGNUP_REDIRECT_URL)

    def get_email_confirmation_url(self, request, emailconfirmation):
        """Constructs the email confirmation (activation) url.
        Note that if you have architected your system such that email
        confirmations are sent outside of the request context `request`
        can be `None` here.
        """
        url = reverse_lazy("accounts:confirm_email",
                           args=[emailconfirmation.key])
        ret = build_absolute_uri(request, url)
        return ret

    def send_confirmation_mail(self, request, emailconfirmation, signup):
        current_site = get_current_site(request)
        activate_url = self.get_email_confirmation_url(
            request, emailconfirmation)
        ctx = {
            "user": emailconfirmation.email_address.user,
            "activate_url": "Yollo Yollo from accounts",
            "current_site": current_site,
            "key": emailconfirmation.key,
        }
        if signup:
            email_template = "accounts/email/email_confirmation_signup"
        else:
            email_template = "accounts/email/email_confirmation"
        self.send_mail(
            email_template, emailconfirmation.email_address.email, ctx)
        
        
    def send_token_confirmation_mail(self, request, emailconfirmation, signup):
        current_site = get_current_site(request)
        activate_url = self.get_email_confirmation_url(
            request, emailconfirmation)
        ctx = {
            "user": emailconfirmation.email_address.user,
            "activate_url": "Yollo Yollo from accounts",
            "current_site": current_site,
            "key": emailconfirmation.key,
        }
        if signup:
            email_template = "accounts/email/email_confirmation_signup"
        else:
            email_template = "accounts/email/email_confirmation"
        self.send_mail(
            email_template, emailconfirmation.email_address.email, ctx)
        
    
    def render_mail(self, template_prefix, email, context):
        """
        Renders an e-mail to `email`.  `template_prefix` identifies the
        e-mail that is to be sent, e.g. "account/email/email_confirmation"
        """
        to = [email] if isinstance(email, str) else email
        subject = render_to_string("{0}_subject.txt".format(template_prefix), context)
        # remove superfluous line breaks
        subject = " ".join(subject.splitlines()).strip()
        subject = self.format_email_subject(subject)

        from_email = self.get_from_email()

        bodies = {}
        for ext in ["html", "txt"]:
            try:
                template_name = "{0}_message.{1}".format(template_prefix, ext)
                bodies[ext] = render_to_string(
                    template_name,
                    context,
                    self.request,
                ).strip()
            except TemplateDoesNotExist:
                if ext == "txt" and not bodies:
                    # We need at least one body
                    raise
        if "txt" in bodies:
            msg = EmailMultiAlternatives(subject, bodies["txt"], from_email, to)
            if "html" in bodies:
                msg.attach_alternative(bodies["html"], "text/html")
        else:
            msg = EmailMessage(subject, bodies["html"], from_email, to)
            msg.content_subtype = "html"  # Main content is now text/html
        return msg

    def send_mail(self, template_prefix, email, context):
        msg = self.render_mail(template_prefix, email, context)
        msg.send()
        

    def respond_email_verification_sent(self, request, user):
        return HttpResponseRedirect(reverse_lazy("accounts:email_verification_sent"))
